#ifndef _BOARD_H
#define _BOARD_H

#define EMPTY 0 //!< For empty intersection
#define BLACK 1 //!< For an intersection occupied by a black stone
#define WHITE 2 //!< For an intersection occupied by a white stone

#include <glibmm/ustring.h>
#include <iostream>
#include "Error.h"
#include "Coord.h"
#include <list>

using namespace std;
using Glib::ustring;

//! A class representing the game board.
/*!
 * The labels, the size and possibly the board state is stored.
 * The board state is stored as a two dimensional array of char.
 * To spare memory, the board state is not nesseceraly stored
 * (There is one Board object for each game node \see SgfNode).
 * To store it, one
 * must call AllocateBoard() which allocates an empty board
 * and then fill it with the stones. The
 * board state is also allocated when using the assignement operator
 * operator=(const Board& board) if the board to assign from has
 * already an allocated board state.
 * \todo maybe remove the complex memory management and simply replace it
 * with two lists of coord, one for black and one for white. But it has the
 * drawback that it does not allow to simply check if an intersection is occupied.
 */
class Board {
    private:
        char* m_table;
        unsigned int m_size;
        list<Label> m_labels;
        list<Coord> m_black_terr;
        list<Coord> m_white_terr;
        Coord m_ko;
        int m_black_captures;
        int m_white_captures;
    public:
        //! Default constructor
        /*!
         * The size of the board is set to 0.
         */
        Board():m_size(0),m_table(0),m_ko(-1,-1), m_black_captures(0), m_white_captures(0) {}
        //! Constructor
        /*!
         * The board state is not allocated!
         * \param size the size of the board.
         */
        Board(unsigned int size);
        //! Deep copy constructor
        /*!
         * Performs a deep copy. If the board to copy
         * has an allocated board state, a new board
         * state will be allocated and filled with the values
         * of the board state to copy from.
         * \param board the Board to copy from.
         */
        Board(const Board& board); //Deep copy!
        //! Assignation operator.
        /*!
         * Performs a deep copy. If the board to copy
         * has an allocated board state, but not the Board to copy to,
         * a new board state will be allocated and filled with the values
         * of the board state to copy from. If a board state is already allocated,
         * it will be simply filled with the board state values of the Board to copy from.
         * \param board the Board to copy from.
         * \throw BoardError if the sizes are not equal
         * \throw BoardError if the board to assign from has no board state allocated, but to one to assign to has.
         * \todo Do not allow to assign an allocated board state to an unallocated one,
         * so the only way to allocate the board state is using AllocateBoard().
         */
        Board& operator=(const Board& board); //Deep copy!
        //!destructor
        ~Board() {if(m_table) delete [] m_table;}
        //! Allocate the board state.
        /*!
         * \throw BoardError if the size of the board was not defined.
         */
        void AllocateBoard();
        //! Tells if the board state is allocated.
        /*!
         * \return true if the board state is allocated
         */
        bool IsAllocated() const {return m_table;}
        //! Free the board state.
        void FreeBoard();
        //! Get a reference to the content of an intersection
        /*!
         * \param x is the first coordinate.
         * \param y is the second coordinate.
         * \throw BoardError if the coordinates are out of bounds.
         */
        char& operator()(const int& x, const int& y);
        //! Get a reference to the content of an intersection
        /*!
         * \param c the coordinates.
         * \throw BoardError if the coordinates are out of bounds.
         */
        char& operator()(const Coord& c);
        //! Get the content of an intersection
        /*!
         * \param x is the first coordinate.
         * \param y is the second coordinate.
         * \throw BoardError if the coordinates are out of bounds.
         */
        const char& operator()(const int& x, const int& y) const {return m_table[x*m_size+y];}
        //! Get the content of an intersection
        /*!
         * \param c the coordinates.
         * \throw BoardError if the coordinates are out of bounds.
         */
        const char& operator()(const Coord& c) const {return m_table[c.i()*m_size + c.j()];}
        int GetBlackCaptures() const {return m_black_captures;}
        void AddBlackCaptures(int capt) {m_black_captures=capt;}
        int GetWhiteCaptures() const {return m_white_captures;}
        void AddWhiteCaptures(int capt) {m_white_captures=capt;}
        //! Get an iterator to the beginning of the labels list.
        /*!
         * \return an iterator to the beginning of the labels list.
         */
        list<Label>::const_iterator LabelBegin() const {return m_labels.begin();}
        //! Get an iterator to the end of the labels list.
        /*!
         * \return an iterator to the end of the labels list.
         */
        list<Label>::const_iterator LabelEnd() const {return m_labels.end();}
        //! Adds a label.
        /*!
         * Adds a label to the board.
         * \param coord the coordinates where tu put the label.
         * \param type the type of the label.
         * \param letter the letter of the label.
         * \see Label
         */
        void AddLabel(const Coord& coord, Label::Type type, char letter=0) {m_labels.push_back(Label(coord,type,letter));}
        //! Adds black territory coordinates
        void AddBlackTerritory(Coord terr) {m_black_terr.push_back(terr);}
        list<Coord>::const_iterator BlackTerritoryBegin() const {return m_black_terr.begin();}
        list<Coord>::const_iterator BlackTerritoryEnd() const {return m_black_terr.end();}
        //! Adds white territory coordinates
        void AddWhiteTerritory(Coord terr) {m_white_terr.push_back(terr);}
        list<Coord>::const_iterator WhiteTerritoryBegin() const {return m_white_terr.begin();}
        list<Coord>::const_iterator WhiteTerritoryEnd() const {return m_white_terr.end();}
        //!Sets the size of the board.
        /*!
         * \param size the size.
         * \throw BoardError if the board state is already allocated
         */
        void SetSize(unsigned int size);
        //!Set the Ko point ((-1,-1) if none)
        void SetKo(const Coord& ko){m_ko=ko;}
        //!Get the Ko point ((-1,-1) if none)
        Coord GetKo() const {return m_ko;}
        //! Gets the size of the board.
        /*!
         * \return the size of the board.
         */
        unsigned int Size() const {return m_size;}
        //! Print the board state to the console
        friend ostream& operator<<(ostream& os, const Board& board);
};

//! The error class for the Board class
class BoardError: public Error{
    public:
        BoardError(ustring err):Error(err){}
        friend ostream& operator<<(ostream& out,const BoardError& err);
};

#endif /*_BOARD_H*/
